@@ -44,7 +44,11 @@ shared_options = [
                  help='Configure logging verbosity.'),
     click.option('--no-batch',
                  is_flag=True,
-                 help="Don't batch results, write them as they come in.")
+                 help="Don't batch results, write them as they come in."),
+    click.option('--no-cleanup',
+                 is_flag=True,
+                 hidden=True,
+                 help="Hidden developer option, if flagged, don't automatically cleanup results directory on failure.")
 ]
 
 
@@ -69,17 +73,25 @@ def psimulate():
 @psimulate.command()
 @click.argument('model_specification', type=click.Path(exists=True, dir_okay=False))
 @click.argument('branch_configuration', type=click.Path(exists=True, dir_okay=False))
+@click.option('--artifact_path', '-i', type=click.Path(resolve_path=True), help='The path to the artifact data file.')
 @click.option('--result-directory', '-o', type=click.Path(exists=True, file_okay=False), default=None,
               help='The directory to write results to. A folder will be created in this directory with the same name '
                    'as the configuration file.')
 @pass_shared_options
-def run(model_specification, branch_configuration, result_directory, **options):
+def run(model_specification, branch_configuration, artifact_path, result_directory, **options):
     """Run a parallel simulation.
 
     The simulation itself is defined by a MODEL_SPECIFICATION yaml file
     and the parameter changes across runs are defined by a BRANCH_CONFIGURATION
     yaml file.
 
+    The path to the data artifact can be provided as an argument here, in the
+    branch configuration, or in the model specification file. Values provided as
+    a command line argument or in the branch specification file will override a
+    value specified in the model specifications file. If an artifact path is
+    provided both as a command line argument and to the branch configuration file
+    a ConfigurationError will be thrown.
+
     If a results directory is provided, a subdirectory will be created with the
     same name as the MODEL_SPECIFICATION if one does not exist. Results will be
     written to a further subdirectory named after the start time of the
@@ -92,12 +104,12 @@ def run(model_specification, branch_configuration, result_directory, **options):
     utilities.configure_master_process_logging_to_terminal(options['verbose'])
     main = handle_exceptions(runner.main, logger, options['with_debugger'])
 
-    main(model_specification, branch_configuration, result_directory,
+    main(model_specification, branch_configuration, artifact_path, result_directory,
          {'project': options['project'],
           'queue': options['queue'],
           'peak_memory': options['peak_memory'],
           'max_runtime': options['max_runtime']},
-         redis_processes=options['redis'], no_batch=options['no_batch'])
+         redis_processes=options['redis'], no_batch=options['no_batch'], no_cleanup=options['no_cleanup'])
 
 
 @psimulate.command()
@@ -114,12 +126,12 @@ def restart(results_root, **options):
     utilities.configure_master_process_logging_to_terminal(options['verbose'])
     main = handle_exceptions(runner.main, logger, options['with_debugger'])
 
-    main(None, None, results_root,
+    main(None, None, None, results_root,
          {'project': options['project'],
           'queue': options['queue'],
           'peak_memory': options['peak_memory'],
           'max_runtime': options['max_runtime']},
-         redis_processes=options['redis'], restart=True, no_batch=options['no_batch'])
+         redis_processes=options['redis'], restart=True, no_batch=options['no_batch'], no_cleanup=options['no_cleanup'])
 
 
 @psimulate.command()
@@ -140,7 +152,7 @@ def expand(results_root, **options):
     utilities.configure_master_process_logging_to_terminal(options['verbose'])
     main = handle_exceptions(runner.main, logger, options['with_debugger'])
 
-    main(None, None, results_root,
+    main(None, None, None, results_root,
          {'project': options['project'],
           'queue': options['queue'],
           'peak_memory': options['peak_memory'],
@@ -149,4 +161,5 @@ def expand(results_root, **options):
          restart=True,
          expand={'num_draws': options['add_draws'],
                  'num_seeds': options['add_seeds']},
-         no_batch=options['no_batch'])
+         no_batch=options['no_batch'],
+         no_cleanup=options['no_cleanup'])
