@@ -8,26 +8,31 @@ import subprocess
 from typing import List, Dict, Tuple
 from pathlib import Path
 from time import sleep, time
+import yaml
 
 import numpy as np
 import pandas as pd
 from loguru import logger
 
-from vivarium.framework.configuration import build_model_specification
+from vivarium.framework.configuration import build_model_specification, ConfigurationError
 from vivarium.framework.utilities import collapse_nested_dict
 from vivarium.framework.artifact import parse_artifact_path_config
 
 from vivarium_cluster_tools.psimulate.branches import Keyspace
 from vivarium_cluster_tools.psimulate import globals as vct_globals, utilities
 from vivarium_cluster_tools.psimulate.registry import RegistryManager
+from vivarium_cluster_tools.vipin.perf_report import report_performance
 
 drmaa = utilities.get_drmaa()
 
+MODEL_SPEC_FILENAME = 'model_specification.yaml'
+FULL_ARTIFACT_PATH_KEY = f'{vct_globals.INPUT_DATA_KEY}.{vct_globals.ARTIFACT_PATH_KEY}'
+
 
 class RunContext:
-    def __init__(self, model_specification_file: str, branch_configuration_file: str, output_directory: Path,
-                 logging_directories: Dict[str, Path], num_input_draws: int, num_random_seeds: int,
-                 restart: bool, expand: Dict[str, int], no_batch: bool):
+    def __init__(self, model_specification_file: str, branch_configuration_file: str, artifact_path: str,
+                 output_directory: Path, logging_directories: Dict[str, Path], num_input_draws: int,
+                 num_random_seeds: int, restart: bool, expand: Dict[str, int], no_batch: bool):
         self.number_already_completed = 0
         self.output_directory = output_directory
         self.no_batch = no_batch
@@ -46,25 +51,29 @@ class RunContext:
 
             self.keyspace = Keyspace.from_branch_configuration(num_input_draws, num_random_seeds,
                                                                branch_configuration_file)
+            if artifact_path:
+                if FULL_ARTIFACT_PATH_KEY in self.keyspace:
+                    raise ConfigurationError('Artifact path cannot be specified both in the branch specification file'
+                                             ' and as a command line argument.', artifact_path)
+                if not Path(artifact_path).exists():
+                    raise FileNotFoundError(f"Cannot find artifact at path {artifact_path}")
 
-            if "input_data.artifact_path" in self.keyspace.get_data():
-                raise ValueError("An artifact path can only be supplied in the model specification file, "
-                                 "not the branches configuration.")
-
-            if "artifact_path" in model_specification.configuration.input_data:
+            elif vct_globals.ARTIFACT_PATH_KEY in model_specification.configuration[vct_globals.INPUT_DATA_KEY]:
                 artifact_path = parse_artifact_path_config(model_specification.configuration)
-                model_specification.configuration.input_data.update(
-                    {"artifact_path": artifact_path},
+
+            if artifact_path:
+                model_specification.configuration[vct_globals.INPUT_DATA_KEY].update(
+                    {vct_globals.ARTIFACT_PATH_KEY: artifact_path},
                     source=__file__)
 
-            model_specification_path = self.output_directory / 'model_specification.yaml'
-            shutil.copy(model_specification_file, model_specification_path)
+            with open(self.output_directory / MODEL_SPEC_FILENAME, 'w') as config_file:
+                yaml.dump(model_specification.to_dict(), config_file)
 
             self.existing_outputs = None
 
             # Log some basic stuff about the simulation to be run.
             self.keyspace.persist(self.output_directory)
-        self.model_specification = self.output_directory / 'model_specification.yaml'
+        self.model_specification = self.output_directory / MODEL_SPEC_FILENAME
 
 
 class NativeSpecification:
@@ -144,7 +153,9 @@ def init_job_template(jt, native_specification: NativeSpecification, sge_log_dir
     export VIVARIUM_LOGGING_DIRECTORY={worker_log_directory}
     export PYTHONPATH={output_dir}:$PYTHONPATH
 
-    {shutil.which('rq')} worker -c {worker_settings_file.stem} --name ${{JOB_ID}}.${{SGE_TASK_ID}} --burst -w "vivarium_cluster_tools.psimulate.distributed_worker.ResilientWorker" --exception-handler "vivarium_cluster_tools.psimulate.distributed_worker.retry_handler" vivarium
+    {shutil.which('rq')} worker -c {worker_settings_file.stem} --name ${{JOB_ID}}.${{SGE_TASK_ID}} --burst \
+        -w "vivarium_cluster_tools.psimulate.distributed_worker.ResilientWorker" \
+        --exception-handler "vivarium_cluster_tools.psimulate.distributed_worker.retry_handler" vivarium
 
     ''')
     launcher.close()
@@ -281,7 +292,7 @@ def build_job_list(ctx: RunContext) -> List[dict]:
 
 def concat_preserve_types(df_list: List[pd.DataFrame]) -> pd.DataFrame:
     """Concatenation preserves all ``numpy`` dtypes but does not preserve any
-    pandas speciifc dtypes (e.g., categories become objects."""
+    pandas specific dtypes (e.g., categories become objects."""
     dtypes = df_list[0].dtypes
     columns_by_dtype = [list(dtype_group.index) for _, dtype_group in dtypes.groupby(dtypes)]
 
@@ -322,7 +333,7 @@ def write_results_batch(ctx: RunContext, written_results: pd.DataFrame, unwritte
             results_to_write.to_hdf(temp_output_path, 'data')
             temp_output_path.replace(output_path)
             break
-        except Exception as e:
+        except Exception:
             logger.warning(f'Error trying to write results to hdf, retries remaining {retries}')
             sleep(30)
             retries -= 1
@@ -383,16 +394,27 @@ def check_user_sge_config():
                                    "with the worker logs.")
 
 
-def main(model_specification_file: str, branch_configuration_file: str, result_directory: str,
+def try_run_vipin(log_path: Path):
+
+    try:
+        report_performance(input_directory=log_path, output_directory=log_path, output_hdf=False, verbose=1)
+    except Exception as e:
+        logger.warning(f'Performance reporting failed with: {e}')
+
+
+def main(model_specification_file: str, branch_configuration_file: str, artifact_path: str, result_directory: str,
          native_specification: dict, redis_processes: int, num_input_draws: int = None,
          num_random_seeds: int = None, restart:  bool = False,
-         expand: Dict[str, int] = None, no_batch: bool = False):
+         expand: Dict[str, int] = None, no_batch: bool = False, no_cleanup: bool = False):
 
     utilities.exit_if_on_submit_host(utilities.get_hostname())
 
     output_dir, logging_dirs = utilities.setup_directories(model_specification_file, result_directory,
                                                            restart, expand=bool(num_input_draws or num_random_seeds))
-    atexit.register(utilities.check_for_empty_results_dir, output_dir=output_dir)
+
+    if not no_cleanup:
+        atexit.register(utilities.check_for_empty_results_dir, output_dir=output_dir)
+
     atexit.register(lambda: logger.remove())
 
     native_specification['job_name'] = output_dir.parts[-2]
@@ -401,7 +423,7 @@ def main(model_specification_file: str, branch_configuration_file: str, result_d
     utilities.configure_master_process_logging_to_file(logging_dirs['main'])
     utilities.validate_environment(output_dir)
 
-    ctx = RunContext(model_specification_file, branch_configuration_file, output_dir, logging_dirs,
+    ctx = RunContext(model_specification_file, branch_configuration_file, artifact_path, output_dir, logging_dirs,
                      num_input_draws, num_random_seeds, restart, expand, no_batch)
     check_user_sge_config()
     jobs = build_job_list(ctx)
@@ -435,4 +457,6 @@ def main(model_specification_file: str, branch_configuration_file: str, result_d
 
     process_job_results(registry_manager, ctx)
 
+    try_run_vipin(logging_dirs['worker'])
+
     logger.info('Jobs completed. Results written to: {}'.format(ctx.output_directory))
